@@ -51,7 +51,7 @@
    <gw-function>
    c-name
    argument-count input-argument-count optional-argument-count
-   arguments argument-types
+   arguments argument-types argument-typespecs
    return-type return-typespec
    generic-name 
    
@@ -106,12 +106,15 @@
 ;;; Conditions
 
 (define-class &gw-bad-typespec (&error &message)
-  (spec #:accessor typespec-form #:init-value #f)
-  (type #:accessor type #:init-value #f)
-  (options #:accessor typespec-options #:init-value #f))
+  (spec #:getter typespec-form #:init-value #f)
+  (type #:getter type #:init-value #f)
+  (options #:getter typespec-options #:init-value #f))
+
+(define-class &gw-bad-typespec-option (&error &message)
+  (option #:getter typespec-option))
 
 (define-class &gw-stacked (&message)
-  (next #:accessor next-condition))
+  (next #:getter next-condition))
 
 (define-method (format-error msg . args)
   (display "g-wrap: " (current-error-port))
@@ -201,6 +204,12 @@
            (spec spec)
            (message (apply format #f msg args))))))
 
+(define-method (raise-bad-typespec-option option (msg <string>) . args)
+  (raise (condition
+          (&gw-bad-typespec-option
+           (option option)
+           (message (apply format #f msg args))))))
+
 (define-method (raise-stacked (next &condition) (msg <string>) . args)
   (raise (condition
           (&gw-stacked
@@ -309,17 +318,21 @@
 
 (define-method (make-typespec (type <gw-type>) (options <list>))
   (check-typespec-options type options)
-  (let ((typespec (make <gw-typespec> #:type type)))
-    (for-each (lambda (opt) (parse-typespec-option! typespec type opt))
-              options)
-    typespec))
+  (guard
+   (c
+    ((is-a? c &gw-bad-typespec-option)
+     (raise-bad-typespec type options "bad typespec option ~S: ~A"
+                         (typespec-option c)
+                         (condition-message c))))
+   (let ((typespec (make <gw-typespec> #:type type)))
+     (for-each (lambda (opt) (parse-typespec-option! typespec type opt))
+               options)
+     typespec)))
 
 (define-method (check-typespec-options (type <gw-type>) (options <list>))
   (if (not (null? options))
-      (raise (condition
-              (&gw-bad-typespec
-               (type type) (options options)
-               (message "typespec may not have options by default"))))))
+      (raise-bad-typespec type options
+                          "typespec may not have options by default")))
 
 (define-method (parse-typespec-option! (typespec <gw-typespec>)
                                        (type <gw-type>)
@@ -329,10 +342,7 @@
 (define-method (parse-typespec-option! (typespec <gw-typespec>)
                                        (type <gw-type>)
                                        option)
-  (raise (condition
-          (&gw-bad-typespec
-           (type type) (options options)
-           (message "typespec options must be symbols")))))
+  (raise-bad-typespec-option option "typespec options must be symbols"))
 
 (define-generic c-type-name)
 
@@ -368,14 +378,17 @@
          (slot-ref func 'arguments)))
 
 ;; Returns the number of optional argument (number of consecutive
-;; arguments with default values at the end of the argument list).  We
-;; count invisible arguments at the argument tail as optional, too.
+;; arguments with default values at the end of the argument list).
 (define-method (optional-argument-count (func <gw-function>))
   (let loop ((args (reverse (slot-ref func 'arguments))) (count 0))
-    (if (or (null? args) (and (visible? (car args)) 
-                              (not (default-value (car args)))))
-        count
-        (loop (cdr args) (+ count 1)))))
+    (cond ((or (null? args) (and (visible? (car args)) 
+                                 (not (default-value (car args)))))
+           count)
+          ((and (visible? (car args))
+                (not (memq 'out (options (typespec (car args))))))
+           (loop (cdr args) (+ count 1)))
+          (else
+           (loop (cdr args) count)))))
 
 (define-method (argument-types (func <gw-function>))
   (map type (slot-ref func 'arguments)))
@@ -562,10 +575,7 @@
   (let* ((form (cond
                 ((symbol? spec) (list spec))
                 ((list? spec) spec)
-                (else (raise (condition
-                              (&gw-bad-typespec
-                               (spec spec)
-                               (message "neither list nor symbol")))))))
+                (else (raise-bad-typespec spec "neither list nor symbol"))))
          (type (lookup-type wrapset (car form))))
     (if type
         (make-typespec
