@@ -24,6 +24,12 @@ long gw_test_out_args(int arg1, int *arg2, char **arg3)
   return arg1;
 }
 
+long gw_test_out_plus_default_args(int arg1, int arg2, char **arg3)
+{
+  *arg3 = "foo";
+  return arg1 * arg2;
+}
+
 /*======================================================================*/
 /* Generics */
 
@@ -59,6 +65,11 @@ void gw_test_generic__double_double_ptr (double d, double *pd)
   *pd = d;
 }
 
+int gw_test_generic__bool_bool (int b1, int b2)
+{
+  return b1 && b2;
+}
+
 /*======================================================================*/
 /* For test-gw-enumeration */
 
